@@ -1,12 +1,12 @@
 """Type inference constraints."""
 
-from typing import List, cast, Undefined
+from typing import List, cast
 
 from mypy.types import (
-    Callable, Type, TypeVisitor, UnboundType, AnyType, Void, NoneTyp, TypeVar,
-    Instance, TupleType, UnionType, Overloaded, ErasedType, is_named_instance
+    CallableType, Type, TypeVisitor, UnboundType, AnyType, Void, NoneTyp, TypeVarType,
+    Instance, TupleType, UnionType, Overloaded, ErasedType, PartialType, DeletedType,
+    is_named_instance
 )
-from mypy.expandtype import expand_caller_var_args
 from mypy.maptype import map_instance_to_supertype
 from mypy import nodes
 import mypy.subtypes
@@ -24,8 +24,12 @@ class Constraint:
 
     type_var = 0   # Type variable id
     op = 0         # SUBTYPE_OF or SUPERTYPE_OF
+    target = None  # type: Type
 
-    target = Undefined(Type)
+    def __init__(self, type_var: int, op: int, target: Type) -> None:
+        self.type_var = type_var
+        self.op = op
+        self.target = target
 
     def __repr__(self) -> str:
         op_str = '<:'
@@ -33,14 +37,9 @@ class Constraint:
             op_str = ':>'
         return '{} {} {}'.format(self.type_var, op_str, self.target)
 
-    def __init__(self, type_var: int, op: int, target: Type) -> None:
-        self.type_var = type_var
-        self.op = op
-        self.target = target
-
 
 def infer_constraints_for_callable(
-        callee: Callable, arg_types: List[Type], arg_kinds: List[int],
+        callee: CallableType, arg_types: List[Type], arg_kinds: List[int],
         formal_to_actual: List[List[int]]) -> List[Constraint]:
     """Infer type variable constraints for a callable and actual arguments.
 
@@ -127,7 +126,8 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
     """Visitor class for inferring type constraints."""
 
     # The type that is compared against a template
-    actual = Undefined(Type)
+    # TODO: The value may be None. Is that actually correct?
+    actual = None  # type: Type
 
     def __init__(self, actual: Type, direction: int) -> None:
         # Direction must be SUBTYPE_OF or SUPERTYPE_OF.
@@ -151,10 +151,22 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
     def visit_erased_type(self, template: ErasedType) -> List[Constraint]:
         return []
 
+    def visit_deleted_type(self, template: DeletedType) -> List[Constraint]:
+        return []
+
+    # Errors
+
+    def visit_partial_type(self, template: PartialType) -> List[Constraint]:
+        # We can't do anything useful with a partial type here.
+        assert False, "Internal error"
+
     # Non-trivial leaf type
 
-    def visit_type_var(self, template: TypeVar) -> List[Constraint]:
-        return [Constraint(template.id, self.direction, self.actual)]
+    def visit_type_var(self, template: TypeVarType) -> List[Constraint]:
+        if self.actual:
+            return [Constraint(template.id, self.direction, self.actual)]
+        else:
+            return []
 
     # Non-leaf types
 
@@ -202,17 +214,20 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
         else:
             return []
 
-    def visit_callable(self, template: Callable) -> List[Constraint]:
-        if isinstance(self.actual, Callable):
-            cactual = cast(Callable, self.actual)
+    def visit_callable_type(self, template: CallableType) -> List[Constraint]:
+        if isinstance(self.actual, CallableType):
+            cactual = cast(CallableType, self.actual)
             # FIX verify argument counts
             # FIX what if one of the functions is generic
             res = []  # type: List[Constraint]
-            for i in range(len(template.arg_types)):
-                # Negate constraints due function argument type contravariance.
-                res.extend(negate_constraints(infer_constraints(
-                    template.arg_types[i], cactual.arg_types[i],
-                    self.direction)))
+
+            # We can't infer constraints from arguments if the template is Callable[..., T] (with
+            # literal '...').
+            if not template.is_ellipsis_args:
+                # The lengths should match, but don't crash (it will error elsewhere).
+                for t, a in zip(template.arg_types, cactual.arg_types):
+                    # Negate constraints due function argument type contravariance.
+                    res.extend(negate_constraints(infer_constraints(t, a, self.direction)))
             res.extend(infer_constraints(template.ret_type, cactual.ret_type,
                                          self.direction))
             return res
@@ -229,7 +244,7 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
             return []
 
     def infer_against_overloaded(self, overloaded: Overloaded,
-                                 template: Callable) -> List[Constraint]:
+                                 template: CallableType) -> List[Constraint]:
         # Create constraints by matching an overloaded type against a template.
         # This is tricky to do in general. We cheat by only matching against
         # the first overload item, and by only matching the return type. This
@@ -266,6 +281,12 @@ class ConstraintBuilderVisitor(TypeVisitor[List[Constraint]]):
             res.extend(infer_constraints(t, AnyType(), self.direction))
         return res
 
+    def visit_overloaded(self, type: Overloaded) -> List[Constraint]:
+        res = []  # type: List[Constraint]
+        for t in type.items():
+            res.extend(infer_constraints(t, self.actual, self.direction))
+        return res
+
 
 def negate_constraints(constraints: List[Constraint]) -> List[Constraint]:
     res = []  # type: List[Constraint]
@@ -285,7 +306,7 @@ def neg_op(op: int) -> int:
         raise ValueError('Invalid operator {}'.format(op))
 
 
-def find_matching_overload_item(overloaded: Overloaded, template: Callable) -> Callable:
+def find_matching_overload_item(overloaded: Overloaded, template: CallableType) -> CallableType:
     """Disambiguate overload item against a template."""
     items = overloaded.items()
     for item in items:
