@@ -8,8 +8,9 @@ This module can be run as a script (lex.py FILE).
 
 import re
 
-from mypy.util import short_type
-from typing import List, Undefined, Function, Dict, Any, Match, Pattern, Set
+from mypy.util import short_type, find_python_encoding
+from mypy import defaults
+from typing import List, Callable, Dict, Any, Match, Pattern, Set, Union, Tuple
 
 
 class Token:
@@ -117,6 +118,10 @@ class Colon(Token):
     pass
 
 
+class EllipsisToken(Token):
+    pass
+
+
 class Op(Token):
     """Operator (e.g. '+' or 'in')"""
 
@@ -128,38 +133,42 @@ class Bom(Token):
 class LexError(Token):
     """Lexer error token"""
 
-    def __init__(self, string: str, type: int) -> None:
+    def __init__(self, string: str, type: int, message: str = None) -> None:
         """Initialize token.
 
         The type argument is one of the error types below.
         """
         super().__init__(string)
         self.type = type
+        self.message = message
+
+    def __str__(self):
+        if self.message:
+            return 'LexError(%s)' % self.message
+        else:
+            return super().__str__()
 
 
 # Lexer error types
 NUMERIC_LITERAL_ERROR = 0
 UNTERMINATED_STRING_LITERAL = 1
 INVALID_CHARACTER = 2
-NON_ASCII_CHARACTER_IN_COMMENT = 3
-NON_ASCII_CHARACTER_IN_STRING = 4
-INVALID_UTF8_SEQUENCE = 5
-INVALID_BACKSLASH = 6
-INVALID_DEDENT = 7
-
-# Encoding contexts
-STR_CONTEXT = 1
-COMMENT_CONTEXT = 2
+DECODE_ERROR = 3
+INVALID_BACKSLASH = 4
+INVALID_DEDENT = 5
 
 
-def lex(string: str, first_line: int = 1, pyversion: int = 3) -> List[Token]:
-    """Analyze string and return an array of token objects.
+def lex(string: Union[str, bytes], first_line: int = 1,
+        pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
+        is_stub_file: bool = False) -> Tuple[List[Token], Set[int]]:
+    """Analyze string, and return an array of token objects and the lines to ignore.
 
-    The last token is always Eof.
+    The last token is always Eof. The intention is to ignore any
+    semantic and type check errors on the ignored lines.
     """
-    l = Lexer(pyversion)
+    l = Lexer(pyversion, is_stub_file=is_stub_file)
     l.lex(string, first_line)
-    return l.tok
+    return l.tok, l.ignored_lines
 
 
 # Reserved words (not including operators)
@@ -172,7 +181,7 @@ keywords_common = set([
 # Reserved words specific for Python version 2
 # TODO (jukka): 'print' should be here, but it breaks the parsing of Python 2
 #               builtins, since they also define the function 'print'.
-keywords2 = set([]) # type: Set[str]
+keywords2 = set([])  # type: Set[str]
 
 # Reserved words specific for Python version 3
 keywords3 = set(['nonlocal'])
@@ -181,26 +190,19 @@ keywords3 = set(['nonlocal'])
 alpha_operators = set(['in', 'is', 'not', 'and', 'or'])
 
 # String literal prefixes
-str_prefixes = set(['r', 'b', 'br', 'u', 'ur'])
+str_prefixes = set(['r', 'b', 'br', 'rb', 'u', 'ur', 'R', 'B', 'U'])
 
 # List of regular expressions that match non-alphabetical operators
 operators = [re.compile('[-+*/<>.%&|^~]'),
-             re.compile('==|!=|<=|>=|\\*\\*|//|<<|>>')]
+             re.compile('==|!=|<=|>=|\\*\\*|//|<<|>>|<>')]
 
 # List of regular expressions that match punctuator tokens
-punctuators = [re.compile('[=,()@]|(->)'),
+punctuators = [re.compile('[=,()@`]|(->)'),
                re.compile('\\['),
                re.compile(']'),
                re.compile('([-+*/%&|^]|\\*\\*|//|<<|>>)=')]
 
 
-# Source file encodings
-DEFAULT_ENCODING = 0
-ASCII_ENCODING = 1
-LATIN1_ENCODING = 2
-UTF8_ENCODING = 3
-
-
 # Map single-character string escape sequences to corresponding characters.
 escape_map = {'a': '\x07',
               'b': '\x08',
@@ -234,7 +236,7 @@ def _parse_str_literal(string: str) -> str:
     s = string[len(prefix):]
     if s.startswith("'''") or s.startswith('"""'):
         return s[3:-3]
-    elif 'r' in prefix:
+    elif 'r' in prefix or 'R' in prefix:
         return s[1:-1].replace('\\' + s[0], s[0])
     else:
         return escape_re.sub(lambda m: escape_repl(m, prefix), s[1:-1])
@@ -255,14 +257,14 @@ def escape_repl(m: Match[str], prefix: str) -> str:
         return chr(int(seq[1:], 16))
     elif seq.startswith('u'):
         # Unicode sequence \uNNNN.
-        if 'b' not in prefix:
+        if 'b' not in prefix and 'B' not in prefix:
             return chr(int(seq[1:], 16))
         else:
             return '\\' + seq
     else:
         # Octal sequence.
         ord = int(seq, 8)
-        if 'b' in prefix:
+        if 'b' in prefix and 'B' in prefix:
             # Make sure code is no larger than 255 for bytes literals.
             ord = ord % 256
         return chr(ord)
@@ -275,28 +277,38 @@ class Lexer:
     s = ''     # The string being analyzed
     line = 0   # Current line number
     pre_whitespace = ''     # Whitespace and comments before the next token
-    enc = DEFAULT_ENCODING  # Encoding TODO implement properly
+    enc = ''                # Encoding
 
     # Generated tokens
-    tok = Undefined(List[Token])
+    tok = None  # type: List[Token]
 
     # Table from byte character value to lexer method. E.g. entry at ord('0')
     # contains the method lex_number().
-    map = Undefined(List[Function[[], None]])
+    map = None  # type: Dict[str, Callable[[], None]]
 
     # Indent levels of currently open blocks, in spaces.
-    indents = Undefined(List[int])
+    indents = None  # type: List[int]
 
     # Open ('s, ['s and {'s without matching closing bracket; used for ignoring
     # newlines within parentheses/brackets.
-    open_brackets = Undefined(List[str])
+    open_brackets = None  # type: List[str]
 
-    def __init__(self, pyversion: int = 3) -> None:
-        self.map = [self.unknown_character] * 256
+    pyversion = defaults.PYTHON3_VERSION
+
+    # Ignore errors on these lines (defined using '# type: ignore').
+    ignored_lines = None  # type: Set[int]
+
+    def __init__(self, pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
+                 is_stub_file: bool = False) -> None:
+        self.map = {}
         self.tok = []
         self.indents = [0]
         self.open_brackets = []
+        self.pyversion = pyversion
+        self.is_stub_file = is_stub_file
+        self.ignored_lines = set()
         # Fill in the map from valid character codes to relevant lexer methods.
+        extra_misc = '' if pyversion[0] >= 3 else '`'
         for seq, method in [('ABCDEFGHIJKLMNOPQRSTUVWXYZ', self.lex_name),
                             ('abcdefghijklmnopqrstuvwxyz_', self.lex_name),
                             ('0123456789', self.lex_number),
@@ -311,38 +323,59 @@ class Lexer:
                             ('\\', self.lex_backslash),
                             ('([{', self.lex_open_bracket),
                             (')]}', self.lex_close_bracket),
-                            ('-+*/<>%&|^~=!,@', self.lex_misc)]:
+                            ('-+*/<>%&|^~=!,@' + extra_misc, self.lex_misc)]:
             for c in seq:
-                self.map[ord(c)] = method
-        if pyversion == 2:
+                self.map[c] = method
+        if pyversion[0] == 2:
             self.keywords = keywords_common | keywords2
-        if pyversion == 3:
-            self.keywords = keywords_common | keywords3
+            # Decimal/hex/octal/binary literal or integer complex literal
+            self.number_exp1 = re.compile('(0[xXoObB][0-9a-fA-F]+|[0-9]+)[lL]?')
 
-    def lex(self, s: str, first_line: int) -> None:
+        if pyversion[0] == 3:
+            self.keywords = keywords_common | keywords3
+            self.number_exp1 = re.compile('0[xXoObB][0-9a-fA-F]+|[0-9]+')
+
+    def lex(self, text: Union[str, bytes], first_line: int) -> None:
         """Lexically analyze a string, storing the tokens at the tok list."""
-        self.s = s
         self.i = 0
         self.line = first_line
 
-        if s.startswith('\xef\xbb\xbf'):
-            self.add_token(Bom(s[0:3]))
+        if isinstance(text, bytes):
+            if text.startswith(b'\xef\xbb\xbf'):
+                self.enc = 'utf8'
+                bom = True
+            else:
+                self.enc, enc_line = find_python_encoding(text, self.pyversion)
+                bom = False
+            try:
+                decoded_text = text.decode(self.enc)
+            except UnicodeDecodeError as err:
+                self.report_unicode_decode_error(err, text)
+                return
+            except LookupError:
+                self.report_unknown_encoding(enc_line)
+                return
+            text = decoded_text
+            if bom:
+                self.add_token(Bom(text[0]))
+        self.s = text
 
         # Parse initial indent; otherwise first-line indent would not generate
         # an error.
         self.lex_indent()
 
-        # Make a local copy of map as a simple optimization.
+        # Use some local variables as a simple optimization.
         map = self.map
+        default = self.unknown_character
 
         # Lex the file. Repeatedly call the lexer method for the current char.
-        while self.i < len(s):
+        while self.i < len(text):
             # Get the character code of the next character to lex.
-            c = ord(s[self.i])
+            c = text[self.i]
             # Dispatch to the relevant lexer method. This will consume some
             # characters in the text, add a token to self.tok and increment
             # self.i.
-            map[c]()
+            map.get(c, default)()
 
         # Append a break if there is no statement/block terminator at the end
         # of input.
@@ -350,7 +383,7 @@ class Lexer:
                                   not isinstance(self.tok[-1], Dedent)):
             self.add_token(Break(''))
 
-        # Attack any dangling comments/whitespace to a final Break token.
+        # Attach any dangling comments/whitespace to a final Break token.
         if self.tok and isinstance(self.tok[-1], Break):
             self.tok[-1].string += self.pre_whitespace
             self.pre_whitespace = ''
@@ -360,13 +393,44 @@ class Lexer:
 
         self.add_token(Eof(''))
 
+    def report_unicode_decode_error(self, exc: UnicodeDecodeError, text: bytes) -> None:
+        lines = text.splitlines()
+        for line in lines:
+            try:
+                line.decode(self.enc)
+            except UnicodeDecodeError as new_exc:
+                exc = new_exc
+                break
+            self.line += 1
+        else:
+            self.line = 1
+        self.add_token(
+            LexError('', DECODE_ERROR,
+                     "%r codec can't decode byte %d in column %d" % (
+                         self.enc, line[exc.start], exc.start + 1)))
+        self.add_token(Break(''))
+        self.add_token(Eof(''))
+
+    def report_unknown_encoding(self, encoding_line: int) -> None:
+        self.line = encoding_line
+        self.add_token(
+            LexError('', DECODE_ERROR,
+                     "Unknown encoding %r" % self.enc))
+        self.add_token(Break(''))
+        self.add_token(Eof(''))
+
     def lex_number_or_dot(self) -> None:
         """Analyse a token starting with a dot.
 
-        It can be the member access operator or a float literal such as '.123'.
+        It can be the member access operator, a float literal such as '.123',
+        or an ellipsis (for Python 3 and for all stub files).
         """
         if self.is_at_number():
             self.lex_number()
+        elif self.is_at_ellipsis():
+            # '...' is valid in Python 2 as a token but it's use is limited to indexing.
+            # Example: Tuple[int, ...] is valid in Python 2.
+            self.lex_ellipsis()
         else:
             self.lex_misc()
 
@@ -376,18 +440,27 @@ class Lexer:
         """Is the current location at a numeric literal?"""
         return self.match(self.number_exp) != ''
 
+    ellipsis_exp = re.compile(r'\.\.\.')
+
+    def is_at_ellipsis(self) -> bool:
+        """Is the current location at a ellipsis '...'"""
+        return self.match(self.ellipsis_exp) != ''
+
     # Regexps used by lex_number
 
-    # Decimal/hex/octal literal or integer complex literal
-    number_exp1 = re.compile('0[xXoO][0-9a-fA-F]+|[0-9]+')
+    # NOTE: number_exp1 depends on Python version and is defined in __init__.
+
     # Float literal, e.g. '1.23' or '12e+34' or '1.2j'
     number_exp2 = re.compile(
         r'[0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+[eE][-+]?[0-9]+')
+
     # Complex literal, e.g. '3j' or '1.5e2J'
     number_complex = re.compile(
         r'([0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+([eE][-+]?[0-9]+)?)[jJ]')
+
     # These characters must not appear after a number literal.
     name_char_exp = re.compile('[a-zA-Z0-9_]')
+    octal_int = re.compile('0+[1-9]')
 
     def lex_number(self) -> None:
         """Analyse an int or float literal.
@@ -407,7 +480,11 @@ class Lexer:
             self.add_token(LexError(' ' * maxlen, NUMERIC_LITERAL_ERROR))
         elif len(s1) == maxlen:
             # Integer literal.
-            self.add_token(IntLit(s1))
+            if self.pyversion[0] >= 3 and self.octal_int.match(s1):
+                # Python 2 style octal literal such as 0377 not supported in Python 3.
+                self.add_token(LexError(s1, NUMERIC_LITERAL_ERROR))
+            else:
+                self.add_token(IntLit(s1))
         elif len(s2) == maxlen:
             # Float literal.
             self.add_token(FloatLit(s2))
@@ -415,6 +492,9 @@ class Lexer:
             # Complex literal.
             self.add_token(ComplexLit(sc))
 
+    def lex_ellipsis(self) -> None:
+        self.add_token(EllipsisToken('...'))
+
     name_exp = re.compile('[a-zA-Z_][a-zA-Z0-9_]*')
 
     def lex_name(self) -> None:
@@ -428,7 +508,7 @@ class Lexer:
             self.add_token(Keyword(s))
         elif s in alpha_operators:
             self.add_token(Op(s))
-        elif s in str_prefixes and self.match(re.compile('[a-z]+[\'"]')) != '':
+        elif s in str_prefixes and self.match(re.compile('[a-zA-Z]+[\'"]')) != '':
             self.lex_prefixed_str(s)
         else:
             self.add_token(Name(s))
@@ -437,13 +517,13 @@ class Lexer:
 
     # Initial part of a single-quoted literal, e.g. b'foo' or b'foo\\\n
     str_exp_single = re.compile(
-        r"[a-z]*'([^'\\\r\n]|\\[^\r\n])*('|\\(\n|\r\n?))")
+        r"[a-zA-Z]*'([^'\\\r\n]|\\[^\r\n])*('|\\(\n|\r\n?))")
     # Non-initial part of a multiline single-quoted literal, e.g. foo'
     str_exp_single_multi = re.compile(
         r"([^'\\\r\n]|\\[^\r\n])*('|\\(\n|\r\n?))")
     # Initial part of a single-quoted raw literal, e.g. r'foo' or r'foo\\\n
     str_exp_raw_single = re.compile(
-        r"[a-z]*'([^'\r\n\\]|\\'|\\[^\n\r])*('|\\(\n|\r\n?))")
+        r"[a-zA-Z]*'([^'\r\n\\]|\\'|\\[^\n\r])*('|\\(\n|\r\n?))")
     # Non-initial part of a raw multiline single-quoted literal, e.g. foo'
     str_exp_raw_single_multi = re.compile(
         r"([^'\r\n]|'')*('|\\(\n|\r\n?))")
@@ -451,7 +531,7 @@ class Lexer:
     # Start of a ''' literal, e.g. b'''
     str_exp_single3 = re.compile("[a-z]*'''")
     # End of a ''' literal, e.g. foo'''
-    str_exp_single3end = re.compile(r"[^\n\r]*?'''")
+    str_exp_single3end = re.compile(r"([^\n\r\\]|\\[^\n\r])*?'''")
 
     # The following are similar to above (but use double quotes).
 
@@ -465,7 +545,7 @@ class Lexer:
         r'([^"\r\n]|"")*("|\\(\n|\r\n?))')
 
     str_exp_double3 = re.compile('[a-z]*"""')
-    str_exp_double3end = re.compile(r'[^\n\r]*?"""')
+    str_exp_double3end = re.compile(r'([^\n\r\\]|\\[^\n\r])*?"""')
 
     def lex_str_single(self) -> None:
         """Analyse single-quoted string literal"""
@@ -479,11 +559,11 @@ class Lexer:
 
     def lex_prefixed_str(self, prefix: str) -> None:
         """Analyse a string literal with a prefix, such as r'...'."""
-        s = self.match(re.compile('[a-z]+[\'"]'))
+        s = self.match(re.compile('[a-zA-Z]+[\'"]'))
         if s.endswith("'"):
             re1 = self.str_exp_single
             re2 = self.str_exp_single_multi
-            if 'r' in prefix:
+            if 'r' in prefix or 'R' in prefix:
                 re1 = self.str_exp_raw_single
                 re2 = self.str_exp_raw_single_multi
             self.lex_str(re1, re2, self.str_exp_single3,
@@ -491,7 +571,7 @@ class Lexer:
         else:
             re1 = self.str_exp_double
             re2 = self.str_exp_double_multi
-            if 'r' in prefix:
+            if 'r' in prefix or 'R' in prefix:
                 re1 = self.str_exp_raw_double
                 re2 = self.str_exp_raw_double_multi
             self.lex_str(re1, re2, self.str_exp_double3,
@@ -517,10 +597,9 @@ class Lexer:
                 if s.endswith('\n') or s.endswith('\r'):
                     self.lex_multiline_string_literal(re2, s)
                 else:
-                    self.verify_encoding(s, STR_CONTEXT)
-                    if 'b' in prefix:
+                    if 'b' in prefix or 'B' in prefix:
                         self.add_token(BytesLit(s))
-                    elif 'u' in prefix:
+                    elif 'u' in prefix or 'U' in prefix:
                         self.add_token(UnicodeLit(s))
                     else:
                         self.add_token(StrLit(s))
@@ -546,10 +625,10 @@ class Lexer:
             ss += s
             self.line += 1
             self.i += len(s)
-        lit = Undefined  # type: Token
-        if 'b' in prefix:
+        lit = None  # type: Token
+        if 'b' in prefix or 'B' in prefix:
             lit = BytesLit(ss + m.group(0))
-        elif 'u' in prefix:
+        elif 'u' in prefix or 'U' in prefix:
             lit = UnicodeLit(ss + m.group(0))
         else:
             lit = StrLit(ss + m.group(0))
@@ -581,7 +660,6 @@ class Lexer:
     def lex_comment(self) -> None:
         """Analyze a comment."""
         s = self.match(self.comment_exp)
-        self.verify_encoding(s, COMMENT_CONTEXT)
         self.add_pre_whitespace(s)
 
     backslash_exp = re.compile(r'\\(\n|\r\n?)')
@@ -701,8 +779,8 @@ class Lexer:
         s = ''
         t = None  # type: Any
         for re_list, type in [(operators, Op), (punctuators, Punct)]:
-            for re in re_list:
-                s2 = self.match(re)
+            for regexp in re_list:
+                s2 = self.match(regexp)
                 if len(s2) > len(s):
                     t = type
                     s = s2
@@ -712,6 +790,12 @@ class Lexer:
             # '='.
             self.add_token(LexError(self.s[self.i], INVALID_CHARACTER))
         else:
+            if s == '<>':
+                if self.pyversion[0] == 2:
+                    s = '!='
+                else:
+                    self.add_token(Op('<'))
+                    s = '>'
             self.add_token(t(s))
 
     def unknown_character(self) -> None:
@@ -744,6 +828,8 @@ class Lexer:
         self.pre_whitespace += s
         self.i += len(s)
 
+    type_ignore_exp = re.compile(r'[ \t]*#[ \t]*type:[ \t]*ignore\b')
+
     def add_token(self, tok: Token) -> None:
         """Store a token.
 
@@ -756,6 +842,11 @@ class Lexer:
                 and not isinstance(tok, Dedent)):
             raise ValueError('Empty token')
         tok.pre = self.pre_whitespace
+        if self.type_ignore_exp.match(tok.pre):
+            delta = 0
+            if '\n' in tok.pre or '\r' in tok.pre:
+                delta += 1
+            self.ignored_lines.add(self.line - delta)
         tok.line = self.line
         self.tok.append(tok)
         self.i += len(tok.string)
@@ -784,25 +875,6 @@ class Lexer:
             t = self.tok[-1]
             return isinstance(t, Break) or isinstance(t, Dedent)
 
-    def verify_encoding(self, string: str, context: int) -> None:
-        """Verify that token is encoded correctly (using the file encoding)."""
-        codec = None  # type: str
-        if self.enc == ASCII_ENCODING:
-            codec = 'ascii'
-        elif self.enc in [UTF8_ENCODING, DEFAULT_ENCODING]:
-            codec = 'utf8'
-        if codec is not None:
-            try:
-                pass  # FIX string.decode(codec)
-            except UnicodeDecodeError:
-                type = INVALID_UTF8_SEQUENCE
-                if self.enc == ASCII_ENCODING:
-                    if context == STR_CONTEXT:
-                        type = NON_ASCII_CHARACTER_IN_STRING
-                    else:
-                        type = NON_ASCII_CHARACTER_IN_COMMENT
-                self.add_token(LexError('', type))
-
 
 if __name__ == '__main__':
     # Lexically analyze a file and dump the tokens to stdout.
@@ -811,6 +883,6 @@ if __name__ == '__main__':
         print('Usage: lex.py FILE')
         sys.exit(2)
     fnam = sys.argv[1]
-    s = open(fnam).read()
+    s = open(fnam, 'rb').read()
     for t in lex(s):
         print(t)
