@@ -3,9 +3,9 @@
 from typing import List, Tuple, Union, cast
 
 from mypy.types import (
-    Type, UnboundType, TupleType, UnionType, TypeList, AnyType, Callable, StarType
+    Type, UnboundType, TupleType, UnionType, TypeList, AnyType, CallableType, StarType,
+    EllipsisType
 )
-from mypy.typerepr import CommonTypeRepr, ListTypeRepr
 from mypy.lex import Token, Name, StrLit, Break, lex
 from mypy import nodes
 
@@ -59,6 +59,8 @@ class TypeParser:
             return self.parse_type_list()
         elif t.string == '*':
             return self.parse_star_type()
+        elif t.string == '...':
+            return self.parse_ellipsis_type()
         elif isinstance(t, StrLit):
             # Type escaped as string literal.
             typestr = t.parsed()
@@ -96,7 +98,7 @@ class TypeParser:
                 if self.current_token_str() == ')':
                     break
                 items.append(self.parse_type())
-            type = TupleType(items, None, type.line)
+            type = TupleType(items, None, type.line, implicit=True)
         return type
 
     def parse_type_list(self) -> TypeList:
@@ -110,7 +112,7 @@ class TypeParser:
             if self.current_token_str() != ',':
                 break
             commas.append(self.skip())
-        rbracket = self.expect(']')
+        self.expect(']')
         return TypeList(items, line=lbracket.line)
 
     def parse_named_type(self) -> Type:
@@ -127,12 +129,10 @@ class TypeParser:
             components.append(t)
             name += '.' + t.string
 
-        lbracket, rbracket = none, none
         commas = []  # type: List[Token]
         args = []  # type: List[Type]
         if self.current_token_str() == '[':
-            lbracket = self.skip()
-
+            self.skip()
             while True:
                 typ = self.parse_type()
                 args.append(typ)
@@ -140,11 +140,9 @@ class TypeParser:
                     break
                 commas.append(self.skip())
 
-            rbracket = self.expect(']')
+            self.expect(']')
 
-        typ = UnboundType(name, args, line, CommonTypeRepr(components,
-                                                           lbracket,
-                                                           commas, rbracket))
+        typ = UnboundType(name, args, line)
         return typ
 
     def parse_star_type(self) -> Type:
@@ -152,6 +150,10 @@ class TypeParser:
         type = self.parse_type()
         return StarType(type, star.line)
 
+    def parse_ellipsis_type(self) -> Type:
+        ellipsis = self.expect('...')
+        return EllipsisType(ellipsis.line)
+
     # Helpers
 
     def skip(self) -> Token:
@@ -189,14 +191,28 @@ def parse_str_as_type(typestr: str, line: int) -> Type:
     """
 
     typestr = typestr.strip()
-    tokens = lex(typestr, line)
+    tokens = lex(typestr, line)[0]
     result, i = parse_type(tokens, 0)
     if i < len(tokens) - 2:
         raise TypeParseError(tokens[i], i)
     return result
 
 
-def parse_signature(tokens: List[Token]) -> Tuple[Callable, int]:
+def parse_str_as_signature(typestr: str, line: int) -> CallableType:
+    """Parse a signature represented as a string.
+
+    Raise TypeParseError on parse error.
+    """
+
+    typestr = typestr.strip()
+    tokens = lex(typestr, line)[0]
+    result, i = parse_signature(tokens)
+    if i < len(tokens) - 2:
+        raise TypeParseError(tokens[i], i)
+    return result
+
+
+def parse_signature(tokens: List[Token]) -> Tuple[CallableType, int]:
     """Parse signature of form (argtype, ...) -> ...
 
     Return tuple (signature type, token index).
@@ -205,8 +221,8 @@ def parse_signature(tokens: List[Token]) -> Tuple[Callable, int]:
     if tokens[i].string != '(':
         raise TypeParseError(tokens[i], i)
     i += 1
-    arg_types = List[Type]()
-    arg_kinds = List[int]()
+    arg_types = []  # type: List[Type]
+    arg_kinds = []  # type: List[int]
     while tokens[i].string != ')':
         if tokens[i].string == '*':
             arg_kinds.append(nodes.ARG_STAR)
@@ -228,7 +244,7 @@ def parse_signature(tokens: List[Token]) -> Tuple[Callable, int]:
         raise TypeParseError(tokens[i], i)
     i += 1
     ret_type, i = parse_type(tokens, i)
-    return Callable(arg_types,
-                    arg_kinds,
-                    [None] * len(arg_types),
-                    ret_type, None), i
+    return CallableType(arg_types,
+                        arg_kinds,
+                        [None] * len(arg_types),
+                        ret_type, None), i
