@@ -3,8 +3,7 @@
 import typing
 
 from mypy.lex import Token
-from mypy.types import Type, AnyType, NoneTyp, TypeTranslator, TypeVar
-from mypy.typerepr import AnyRepr
+from mypy.types import Type, AnyType, NoneTyp, TypeTranslator, TypeVarType
 
 
 def replace_type_vars(typ: Type, func_tvars: bool = True) -> Type:
@@ -23,13 +22,10 @@ class ReplaceTypeVarsVisitor(TypeTranslator):
     def __init__(self, func_tvars: bool) -> None:
         self.func_tvars = func_tvars
 
-    def visit_type_var(self, t: TypeVar) -> Type:
+    def visit_type_var(self, t: TypeVarType) -> Type:
         if t.id > 0 or self.func_tvars:
-            if t.repr is not None:
-                # Give a representation for the dynamic type.
-                tok = Token('Any')
-                tok.pre = t.repr.name.pre
-                return AnyType(t.line, AnyRepr(tok))
+            if t.line is not None:
+                return AnyType(t.line)
             else:
                 return AnyType()
         else:
@@ -45,7 +41,7 @@ class ReplaceFuncTypeVarsVisitor(TypeTranslator):
     def __init__(self, target_type: Type) -> None:
         self.target_type = target_type
 
-    def visit_type_var(self, t: TypeVar) -> Type:
+    def visit_type_var(self, t: TypeVarType) -> Type:
         if t.id < 0:
             return self.target_type
         else:
