@@ -1,20 +1,21 @@
 """Generic node traverser visitor"""
 
-from typing import typevar, Generic
+from typing import TypeVar, Generic
 
 from mypy.visitor import NodeVisitor
 from mypy.nodes import (
-    Block, MypyFile, VarDef, FuncItem, CallExpr, ClassDef, Decorator, FuncDef,
+    Block, MypyFile, FuncItem, CallExpr, ClassDef, Decorator, FuncDef,
     ExpressionStmt, AssignmentStmt, OperatorAssignmentStmt, WhileStmt,
-    ForStmt, ReturnStmt, AssertStmt, YieldStmt, DelStmt, IfStmt, RaiseStmt,
-    TryStmt, WithStmt, ParenExpr, MemberExpr, OpExpr, SliceExpr, CastExpr,
+    ForStmt, ReturnStmt, AssertStmt, DelStmt, IfStmt, RaiseStmt,
+    TryStmt, WithStmt, MemberExpr, OpExpr, SliceExpr, CastExpr,
     UnaryExpr, ListExpr, TupleExpr, DictExpr, SetExpr, IndexExpr,
     GeneratorExpr, ListComprehension, ConditionalExpr, TypeApplication,
-    FuncExpr, ComparisonExpr, OverloadedFuncDef, YieldFromStmt, YieldFromExpr
+    FuncExpr, ComparisonExpr, OverloadedFuncDef, YieldFromExpr,
+    YieldExpr
 )
 
 
-T = typevar('T')
+T = TypeVar('T')
 
 
 class TraverserVisitor(NodeVisitor[T], Generic[T]):
@@ -37,11 +38,14 @@ class TraverserVisitor(NodeVisitor[T], Generic[T]):
             s.accept(self)
 
     def visit_func(self, o: FuncItem) -> T:
-        for i in o.init:
-            if i is not None:
-                i.accept(self)
-        for v in o.args:
-            self.visit_var(v)
+        for arg in o.arguments:
+            init = arg.initialization_statement
+            if init is not None:
+                init.accept(self)
+
+        for arg in o.arguments:
+            self.visit_var(arg.variable)
+
         o.body.accept(self)
 
     def visit_func_def(self, o: FuncDef) -> T:
@@ -60,12 +64,6 @@ class TraverserVisitor(NodeVisitor[T], Generic[T]):
         for decorator in o.decorators:
             decorator.accept(self)
 
-    def visit_var_def(self, o: VarDef) -> T:
-        if o.init is not None:
-            o.init.accept(self)
-        for v in o.items:
-            self.visit_var(v)
-
     def visit_expression_stmt(self, o: ExpressionStmt) -> T:
         o.expr.accept(self)
 
@@ -99,14 +97,6 @@ class TraverserVisitor(NodeVisitor[T], Generic[T]):
         if o.expr is not None:
             o.expr.accept(self)
 
-    def visit_yield_stmt(self, o: YieldStmt) -> T:
-        if o.expr is not None:
-            o.expr.accept(self)
-
-    def visit_yield_from_stmt(self, o: YieldFromStmt) -> T:
-        if o.expr is not None:
-            o.expr.accept(self)
-
     def visit_del_stmt(self, o: DelStmt) -> T:
         if o.expr is not None:
             o.expr.accept(self)
@@ -139,19 +129,20 @@ class TraverserVisitor(NodeVisitor[T], Generic[T]):
     def visit_with_stmt(self, o: WithStmt) -> T:
         for i in range(len(o.expr)):
             o.expr[i].accept(self)
-            if o.name[i] is not None:
-                o.name[i].accept(self)
+            if o.target[i] is not None:
+                o.target[i].accept(self)
         o.body.accept(self)
 
-    def visit_paren_expr(self, o: ParenExpr) -> T:
-        o.expr.accept(self)
-
     def visit_member_expr(self, o: MemberExpr) -> T:
         o.expr.accept(self)
 
     def visit_yield_from_expr(self, o: YieldFromExpr) -> T:
         o.expr.accept(self)
 
+    def visit_yield_expr(self, o: YieldExpr) -> T:
+        if o.expr:
+            o.expr.accept(self)
+
     def visit_call_expr(self, o: CallExpr) -> T:
         for a in o.args:
             a.accept(self)
