@@ -7,20 +7,21 @@ from typing import List, Dict, cast
 
 from mypy.nodes import (
     MypyFile, Import, Node, ImportAll, ImportFrom, FuncItem, FuncDef,
-    OverloadedFuncDef, ClassDef, Decorator, Block, Var, VarDef,
+    OverloadedFuncDef, ClassDef, Decorator, Block, Var,
     OperatorAssignmentStmt, ExpressionStmt, AssignmentStmt, ReturnStmt,
-    RaiseStmt, AssertStmt, YieldStmt, DelStmt, BreakStmt, ContinueStmt,
+    RaiseStmt, AssertStmt, DelStmt, BreakStmt, ContinueStmt,
     PassStmt, GlobalDecl, WhileStmt, ForStmt, IfStmt, TryStmt, WithStmt,
-    CastExpr, ParenExpr, TupleExpr, GeneratorExpr, ListComprehension, ListExpr,
+    CastExpr, TupleExpr, GeneratorExpr, ListComprehension, ListExpr,
     ConditionalExpr, DictExpr, SetExpr, NameExpr, IntExpr, StrExpr, BytesExpr,
     UnicodeExpr, FloatExpr, CallExpr, SuperExpr, MemberExpr, IndexExpr,
     SliceExpr, OpExpr, UnaryExpr, FuncExpr, TypeApplication, PrintStmt,
-    SymbolTable, RefExpr, UndefinedExpr, TypeVarExpr, DucktypeExpr,
-    DisjointclassExpr, ComparisonExpr, TempNode, StarExpr, YieldFromStmt,
+    SymbolTable, RefExpr, TypeVarExpr, PromoteExpr,
+    ComparisonExpr, TempNode, StarExpr,
     YieldFromExpr, NamedTupleExpr, NonlocalDecl, SetComprehension,
-    DictionaryComprehension, ComplexExpr
+    DictionaryComprehension, ComplexExpr, TypeAliasExpr, EllipsisExpr,
+    YieldExpr, ExecStmt, Argument, BackquoteExpr
 )
-from mypy.types import Type, FunctionLike
+from mypy.types import Type, FunctionLike, Instance
 from mypy.visitor import NodeVisitor
 
 
@@ -46,11 +47,12 @@ class TransformVisitor(NodeVisitor[Node]):
     def __init__(self) -> None:
         # There may be multiple references to a Var node. Keep track of
         # Var translations using a dictionary.
-        self.var_map = Dict[Var, Var]()
+        self.var_map = {}  # type: Dict[Var, Var]
 
     def visit_mypy_file(self, node: MypyFile) -> Node:
         # NOTE: The 'names' and 'imports' instance variables will be empty!
-        new = MypyFile(self.nodes(node.defs), [], node.is_bom)
+        new = MypyFile(self.nodes(node.defs), [], node.is_bom,
+                       ignored_lines=set(node.ignored_lines))
         new._name = node._name
         new._fullname = node._fullname
         new.path = node.path
@@ -66,12 +68,38 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_import_all(self, node: ImportAll) -> Node:
         return ImportAll(node.id, node.relative)
 
+    def copy_argument(self, argument: Argument) -> Argument:
+        init_stmt = None  # type: AssignmentStmt
+
+        if argument.initialization_statement:
+            init_lvalue = cast(
+                NameExpr,
+                self.node(argument.initialization_statement.lvalues[0]),
+            )
+            init_lvalue.set_line(argument.line)
+            init_stmt = AssignmentStmt(
+                [init_lvalue],
+                self.node(argument.initialization_statement.rvalue),
+                self.optional_type(argument.initialization_statement.type),
+            )
+
+        arg = Argument(
+            self.visit_var(argument.variable),
+            argument.type_annotation,
+            argument.initializer,
+            argument.kind,
+            init_stmt,
+        )
+
+        # Refresh lines of the inner things
+        arg.set_line(argument.line)
+
+        return arg
+
     def visit_func_def(self, node: FuncDef) -> FuncDef:
         # Note that a FuncDef must be transformed to a FuncDef.
         new = FuncDef(node.name(),
-                      [self.visit_var(var) for var in node.args],
-                      node.arg_kinds[:],
-                      [None] * len(node.init),
+                      [self.copy_argument(arg) for arg in node.arguments],
                       self.block(node.body),
                       cast(FunctionLike, self.optional_type(node.type)))
 
@@ -88,9 +116,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return new
 
     def visit_func_expr(self, node: FuncExpr) -> Node:
-        new = FuncExpr([self.visit_var(var) for var in node.args],
-                       node.arg_kinds[:],
-                       [None] * len(node.init),
+        new = FuncExpr([self.copy_argument(arg) for arg in node.arguments],
                        self.block(node.body),
                        cast(FunctionLike, self.optional_type(node.type)))
         self.copy_function_attributes(new, node)
@@ -104,11 +130,10 @@ class TransformVisitor(NodeVisitor[Node]):
         new.is_implicit = original.is_implicit
         new.is_overload = original.is_overload
         new.is_generator = original.is_generator
-        new.init = self.duplicate_inits(original.init)
 
     def duplicate_inits(self,
                         inits: List[AssignmentStmt]) -> List[AssignmentStmt]:
-        result = List[AssignmentStmt]()
+        result = []  # type: List[AssignmentStmt]
         for init in inits:
             if init:
                 result.append(self.duplicate_assignment(init))
@@ -131,22 +156,18 @@ class TransformVisitor(NodeVisitor[Node]):
         new = ClassDef(node.name,
                        self.block(node.defs),
                        node.type_vars,
-                       self.types(node.base_types),
+                       self.nodes(node.base_type_exprs),
                        node.metaclass)
         new.fullname = node.fullname
         new.info = node.info
+        new.base_types = []
+        for base in node.base_types:
+            new.base_types.append(cast(Instance, self.type(base)))
         new.decorators = [decorator.accept(self)
                           for decorator in node.decorators]
         new.is_builtinclass = node.is_builtinclass
         return new
 
-    def visit_var_def(self, node: VarDef) -> Node:
-        new = VarDef([self.visit_var(var) for var in node.items],
-                     node.is_top_level,
-                     self.optional_node(node.init))
-        new.kind = node.kind
-        return new
-
     def visit_global_decl(self, node: GlobalDecl) -> Node:
         return GlobalDecl(node.names[:])
 
@@ -219,12 +240,6 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_assert_stmt(self, node: AssertStmt) -> Node:
         return AssertStmt(self.node(node.expr))
 
-    def visit_yield_stmt(self, node: YieldStmt) -> Node:
-        return YieldStmt(self.node(node.expr))
-
-    def visit_yield_from_stmt(self, node: YieldFromStmt) -> Node:
-        return YieldFromStmt(self.node(node.expr))
-
     def visit_del_stmt(self, node: DelStmt) -> Node:
         return DelStmt(self.node(node.expr))
 
@@ -256,12 +271,18 @@ class TransformVisitor(NodeVisitor[Node]):
 
     def visit_with_stmt(self, node: WithStmt) -> Node:
         return WithStmt(self.nodes(node.expr),
-                        self.optional_names(node.name),
+                        self.optional_nodes(node.target),
                         self.block(node.body))
 
     def visit_print_stmt(self, node: PrintStmt) -> Node:
         return PrintStmt(self.nodes(node.args),
-                         node.newline)
+                         node.newline,
+                         self.optional_node(node.target))
+
+    def visit_exec_stmt(self, node: ExecStmt) -> Node:
+        return ExecStmt(self.node(node.expr),
+                        self.optional_node(node.variables1),
+                        self.optional_node(node.variables2))
 
     def visit_star_expr(self, node: StarExpr) -> Node:
         return StarExpr(node.expr)
@@ -284,8 +305,8 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_complex_expr(self, node: ComplexExpr) -> Node:
         return ComplexExpr(node.value)
 
-    def visit_paren_expr(self, node: ParenExpr) -> Node:
-        return ParenExpr(self.node(node.expr))
+    def visit_ellipsis(self, node: EllipsisExpr) -> Node:
+        return EllipsisExpr()
 
     def visit_name_expr(self, node: NameExpr) -> Node:
         return self.duplicate_name(node)
@@ -318,6 +339,9 @@ class TransformVisitor(NodeVisitor[Node]):
     def visit_yield_from_expr(self, node: YieldFromExpr) -> Node:
         return YieldFromExpr(self.node(node.expr))
 
+    def visit_yield_expr(self, node: YieldExpr) -> Node:
+        return YieldExpr(self.node(node.expr))
+
     def visit_call_expr(self, node: CallExpr) -> Node:
         return CallExpr(self.node(node.callee),
                         self.nodes(node.args),
@@ -367,13 +391,13 @@ class TransformVisitor(NodeVisitor[Node]):
         if node.method_type:
             new.method_type = self.type(node.method_type)
         if node.analyzed:
-            new.analyzed = self.visit_type_application(node.analyzed)
+            if isinstance(node.analyzed, TypeApplication):
+                new.analyzed = self.visit_type_application(node.analyzed)
+            else:
+                new.analyzed = self.visit_type_alias_expr(node.analyzed)
             new.analyzed.set_line(node.analyzed.line)
         return new
 
-    def visit_undefined_expr(self, node: UndefinedExpr) -> Node:
-        return UndefinedExpr(self.type(node.type))
-
     def visit_type_application(self, node: TypeApplication) -> TypeApplication:
         return TypeApplication(self.node(node.expr),
                                self.types(node.types))
@@ -415,18 +439,21 @@ class TransformVisitor(NodeVisitor[Node]):
                                self.node(node.if_expr),
                                self.node(node.else_expr))
 
+    def visit_backquote_expr(self, node: BackquoteExpr) -> Node:
+        return BackquoteExpr(self.node(node.expr))
+
     def visit_type_var_expr(self, node: TypeVarExpr) -> Node:
         return TypeVarExpr(node.name(), node.fullname(),
-                           self.types(node.values))
+                           self.types(node.values), variance=node.variance)
+
+    def visit_type_alias_expr(self, node: TypeAliasExpr) -> TypeAliasExpr:
+        return TypeAliasExpr(node.type)
 
     def visit_namedtuple_expr(self, node: NamedTupleExpr) -> Node:
         return NamedTupleExpr(node.info)
 
-    def visit_ducktype_expr(self, node: DucktypeExpr) -> Node:
-        return DucktypeExpr(node.type)
-
-    def visit_disjointclass_expr(self, node: DisjointclassExpr) -> Node:
-        return DisjointclassExpr(node.cls)
+    def visit__promote_expr(self, node: PromoteExpr) -> Node:
+        return PromoteExpr(node.type)
 
     def visit_temp_node(self, node: TempNode) -> Node:
         return TempNode(self.type(node.type))
@@ -470,7 +497,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return [self.duplicate_name(name) for name in names]
 
     def optional_names(self, names: List[NameExpr]) -> List[NameExpr]:
-        result = List[NameExpr]()
+        result = []  # type: List[NameExpr]
         for name in names:
             if name:
                 result.append(self.duplicate_name(name))
