@@ -1,11 +1,13 @@
 """Generic abstract syntax tree node visitor"""
 
-from typing import typevar, Generic
+from typing import TypeVar, Generic
 
-import mypy.nodes
+if False:
+    # break import cycle only needed for mypy
+    import mypy.nodes
 
 
-T = typevar('T')
+T = TypeVar('T')
 
 
 class NodeVisitor(Generic[T]):
@@ -44,9 +46,6 @@ class NodeVisitor(Generic[T]):
     def visit_class_def(self, o: 'mypy.nodes.ClassDef') -> T:
         pass
 
-    def visit_var_def(self, o: 'mypy.nodes.VarDef') -> T:
-        pass
-
     def visit_global_decl(self, o: 'mypy.nodes.GlobalDecl') -> T:
         pass
 
@@ -86,12 +85,6 @@ class NodeVisitor(Generic[T]):
     def visit_assert_stmt(self, o: 'mypy.nodes.AssertStmt') -> T:
         pass
 
-    def visit_yield_stmt(self, o: 'mypy.nodes.YieldStmt') -> T:
-        pass
-
-    def visit_yield_from_stmt(self, o: 'mypy.nodes.YieldFromStmt') -> T:
-        pass
-
     def visit_del_stmt(self, o: 'mypy.nodes.DelStmt') -> T:
         pass
 
@@ -119,6 +112,9 @@ class NodeVisitor(Generic[T]):
     def visit_print_stmt(self, o: 'mypy.nodes.PrintStmt') -> T:
         pass
 
+    def visit_exec_stmt(self, o: 'mypy.nodes.ExecStmt') -> T:
+        pass
+
     # Expressions
 
     def visit_int_expr(self, o: 'mypy.nodes.IntExpr') -> T:
@@ -139,7 +135,7 @@ class NodeVisitor(Generic[T]):
     def visit_complex_expr(self, o: 'mypy.nodes.ComplexExpr') -> T:
         pass
 
-    def visit_paren_expr(self, o: 'mypy.nodes.ParenExpr') -> T:
+    def visit_ellipsis(self, o: 'mypy.nodes.EllipsisExpr') -> T:
         pass
 
     def visit_star_expr(self, o: 'mypy.nodes.StarExpr') -> T:
@@ -154,6 +150,9 @@ class NodeVisitor(Generic[T]):
     def visit_yield_from_expr(self, o: 'mypy.nodes.YieldFromExpr') -> T:
         pass
 
+    def visit_yield_expr(self, o: 'mypy.nodes.YieldExpr') -> T:
+        pass
+
     def visit_call_expr(self, o: 'mypy.nodes.CallExpr') -> T:
         pass
 
@@ -187,9 +186,6 @@ class NodeVisitor(Generic[T]):
     def visit_index_expr(self, o: 'mypy.nodes.IndexExpr') -> T:
         pass
 
-    def visit_undefined_expr(self, o: 'mypy.nodes.UndefinedExpr') -> T:
-        pass
-
     def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> T:
         pass
 
@@ -214,16 +210,19 @@ class NodeVisitor(Generic[T]):
     def visit_conditional_expr(self, o: 'mypy.nodes.ConditionalExpr') -> T:
         pass
 
+    def visit_backquote_expr(self, o: 'mypy.nodes.BackquoteExpr') -> T:
+        pass
+
     def visit_type_var_expr(self, o: 'mypy.nodes.TypeVarExpr') -> T:
         pass
 
+    def visit_type_alias_expr(self, o: 'mypy.nodes.TypeAliasExpr') -> T:
+        pass
+
     def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> T:
         pass
 
-    def visit_ducktype_expr(self, o: 'mypy.nodes.DucktypeExpr') -> T:
-        pass
-
-    def visit_disjointclass_expr(self, o: 'mypy.nodes.DisjointclassExpr') -> T:
+    def visit__promote_expr(self, o: 'mypy.nodes.PromoteExpr') -> T:
         pass
 
     def visit_temp_node(self, o: 'mypy.nodes.TempNode') -> T:
